from botocore.exceptions import ClientError

from awsassume.data_models import AssumeRoleArgs
from awsassume.security_token_service import SecurityTokenService
from awsassume.data_models import AssumedRoleResponse


class AssumeRole(SecurityTokenService):

    def __init__(self, assume_role_args: AssumeRoleArgs) -> None:
        super().__init__(assume_role_args.region_name)

        self.assume_role_args: AssumeRoleArgs = assume_role_args

    def assume_role(self) -> AssumedRoleResponse:
        try:
            response: AssumedRoleResponse = self.client.assume_role(RoleArn=self.assume_role_args.role_arn,
                                                                    RoleSessionName=self.assume_role_args.role_session_name)
        except ClientError:
            raise
        else:
            return response



from awsassume.assume_role import AssumeRole
from awsassume.assume_role_cache_executor import AssumeRoleCacheExecutor
from awsassume.assume_role_executor import AssumeRoleExecutor
from awsassume.assume_role_no_cache_executor import AssumeRoleNoCacheExecutor
from awsassume.data_models import AssumeRoleArgs, AssumeRoleType, CliArgs, ResponseCacheArgs
from awsassume.response_cache_adapter import ResponseCacheAdapter
from awsassume.security_token_service import SecurityTokenService


class AssumeRoleExecutorFactory(object):

    @staticmethod
    def get_executor(cli_args: CliArgs) -> AssumeRoleExecutor:

        assume_role_args = AssumeRoleArgs(role_arn=cli_args.role_arn, role_session_name=cli_args.role_session_name, region_name=cli_args.region_name)
        security_token_service: SecurityTokenService = AssumeRole(assume_role_args)

        assume_role_executor: AssumeRoleExecutor = None
        if cli_args.no_cache is True:
            assume_role_executor = AssumeRoleNoCacheExecutor(security_token_service)
        else:
            response_cache_args = ResponseCacheArgs(role_arn=cli_args.role_arn,
                                                    role_session_name=cli_args.role_session_name,
                                                    region_name=cli_args.region_name,
                                                    assume_role_type=AssumeRoleType.DEFAULT)

            response_cache_adapter = ResponseCacheAdapter(response_cache_args)
            assume_role_executor = AssumeRoleCacheExecutor(security_token_service, response_cache_adapter)

        return assume_role_executor



import errno
import os
import pickle
import re

from awsassume.data_models import AssumedRoleResponse, ResponseCacheArgs


class ResponseCache(object):
    cache_directory = f'{os.path.expanduser("~")}/.awsassume/cache'

    def __init__(self) -> None:
        try:
            os.makedirs(ResponseCache.cache_directory)
        except OSError as ex:
            if ex.errno != errno.EEXIST:
                raise

    def set_response_to_cache(self, response_cache_args: ResponseCacheArgs, assumed_role_response: AssumedRoleResponse) -> None:
        try:
            with open(self.get_cache_full_path(response_cache_args), 'wb') as file:
                pickle.dump(assumed_role_response, file)
        except OSError:
            raise

    def get_response_from_cache(self, response_cache_args: ResponseCacheArgs) -> AssumedRoleResponse:
        assumed_role_response: AssumedRoleResponse = None

        try:
            with open(self.get_cache_full_path(response_cache_args), 'rb') as file:
                assumed_role_response = pickle.load(file)
        except FileNotFoundError:
            pass
        except OSError:
            raise

        return assumed_role_response

    def delete_cache_file(self, response_cache_args: ResponseCacheArgs) -> None:
        try:
            os.remove(self.get_cache_full_path(response_cache_args))
        except OSError:
            raise

    def get_cache_full_path(self, response_cache_args: ResponseCacheArgs) -> str:
        full_path = f'{ResponseCache.cache_directory}/{self.get_cache_name(response_cache_args)}'

        return full_path

    def get_cache_name(self, response_cache_args: ResponseCacheArgs) -> str:
        cache_name: str = None

        matched_arn = re.match('arn:aws:iam::([0-9]{12}):role/([0-9a-zA-Z+=,.@\-_]{1,64})', response_cache_args.role_arn)
        if matched_arn:
            role_session_name_section = response_cache_args.role_session_name
            role_arn_section = f'{matched_arn.group(1)}_{matched_arn.group(2)}'

            region_name_section: str = 'default'
            if response_cache_args.region_name is not None:
                region_name_section = response_cache_args.region_name

            cache_name = f'{role_session_name_section}__{role_arn_section}__{region_name_section}__{response_cache_args.assume_role_type.name}'

        return cache_name



import subprocess

from typing import Dict, List


class CommandExecutor(object):

    @staticmethod
    def execute(command: List[str], env_var: Dict[str, str]) -> None:
        subprocess.Popen(command, env=env_var)



from datetime import datetime
from enum import auto, Enum
from typing import Any, Dict, List, NamedTuple


class AssumeRoleArgs(NamedTuple):
    role_arn: str
    role_session_name: str
    region_name: str = None


class AssumeRoleType(Enum):
    DEFAULT = auto()


AssumedRoleResponse = Dict[str, Any]


AssumedRoleResponseCredentials = Dict[str, Any]


class CliArgs(NamedTuple):
    role_arn: str
    role_session_name: str
    command: List[str] = None
    region_name: str = None
    no_cache: bool = False


class Credentials(NamedTuple):
    access_key_id: str
    secret_access_key: str
    session_token: str
    expiration: datetime


class ResponseCacheArgs(NamedTuple):
    role_arn: str
    role_session_name: str
    region_name: str = None
    assume_role_type: AssumeRoleType = AssumeRoleType.DEFAULT



import boto3

from abc import ABC, abstractmethod

from awsassume.data_models import AssumedRoleResponse


class SecurityTokenService(ABC):

    def __init__(self, region_name: str = None) -> None:
        self.client = boto3.client('sts', region_name=region_name)

    @abstractmethod
    def assume_role(self) -> AssumedRoleResponse:
        pass



from awsassume.assume_role_executor import AssumeRoleExecutor
from awsassume.assume_role_executor_factory import AssumeRoleExecutorFactory
from awsassume.command_executor import CommandExecutor
from awsassume.command_line_args import CommandLineArgs
from awsassume.data_models import CliArgs, Credentials
from awsassume.environment_variable import EnvironmentVariable


def main():
    cli_args: CliArgs = CommandLineArgs().get_cli_args()

    assume_role_executor: AssumeRoleExecutor = AssumeRoleExecutorFactory.get_executor(cli_args)

    credentials: Credentials = assume_role_executor.execute()

    if cli_args.command is None:
        print('# AWS assumed role credentials:')
        EnvironmentVariable.display_credentials_to_be_exported(credentials, cli_args.region_name)

        print()

        print('# Execute the following command to export the AWS assumed role credentials to environment variables.')

        command_to_eval = EnvironmentVariable.generate_command_to_export_credentials(credentials, cli_args.region_name)
        print(command_to_eval)
    else:
        env_var_with_credentials = EnvironmentVariable.get_env_var_with_credentials(credentials, cli_args.region_name)

        CommandExecutor.execute(cli_args.command, env_var_with_credentials)



import argparse

from typing import Any

from awsassume.data_models import CliArgs


class CommandLineArgs(object):
    def __init__(self) -> None:
        self.parser = argparse.ArgumentParser(description='Execute AWS CLI commands after assuming role')

        required_args: Any = self.parser.add_argument_group('required arguments')
        required_args.add_argument('-a', '--role-arn', required=True, help='The ARN of the role to assume')
        required_args.add_argument('-n', '--role-session-name', required=True, help='An identifier for the assumed role session')

        self.parser.add_argument('--no-cache', required=False, action='store_true', help='Disable caching of the assumed role response')
        self.parser.add_argument('-r', '--region', required=False, help='The region to be associated with the client')
        self.parser.add_argument('-c', '--command', required=False, nargs='+', help='The AWS CLI command to execute after assuming role')

    def get_cli_args(self) -> CliArgs:

        parsed_cli_args: argparse.Namespace = self.parser.parse_args()

        cli_args = CliArgs(role_arn=parsed_cli_args.role_arn,
                           role_session_name=parsed_cli_args.role_session_name,
                           command=parsed_cli_args.command,
                           region_name=parsed_cli_args.region,
                           no_cache=parsed_cli_args.no_cache)

        return cli_args



import os

from typing import Dict

from awsassume.data_models import Credentials


class EnvironmentVariable(object):

    ACCESS_KEY_ID_KEY = 'AWS_ACCESS_KEY_ID'
    SECRET_ACCESS_KEY_KEY = 'AWS_SECRET_ACCESS_KEY'
    SESSION_TOKEN_KEY = 'AWS_SESSION_TOKEN'
    DEFAULT_REGION_KEY = 'AWS_DEFAULT_REGION'

    @staticmethod
    def display_credentials_to_be_exported(credentials: Credentials, region_name: str = None) -> None:

        print(f'# {EnvironmentVariable.ACCESS_KEY_ID_KEY}={credentials.access_key_id}')
        print(f'# {EnvironmentVariable.SECRET_ACCESS_KEY_KEY}={credentials.secret_access_key}')
        print(f'# {EnvironmentVariable.SESSION_TOKEN_KEY}={credentials.session_token}')

        if region_name is not None:
            print(f'# {EnvironmentVariable.DEFAULT_REGION_KEY}={region_name}')

    @staticmethod
    def generate_command_to_export_credentials(credentials: Credentials, region_name: str = None) -> str:

        export_access_key_id = f'export {EnvironmentVariable.ACCESS_KEY_ID_KEY}="{credentials.access_key_id}"'
        export_secret_access_key = f'export {EnvironmentVariable.SECRET_ACCESS_KEY_KEY}="{credentials.secret_access_key}"'
        export_session_token = f'export {EnvironmentVariable.SESSION_TOKEN_KEY}="{credentials.session_token}"'

        command = f'{export_access_key_id}; {export_secret_access_key}; {export_session_token}'

        if region_name is not None:
            export_default_region = f'export {EnvironmentVariable.DEFAULT_REGION_KEY}="{region_name}"'

            command = f'{command}; {export_default_region}'

        return command

    @staticmethod
    def get_env_var_with_credentials(credentials: Credentials, region_name: str = None) -> Dict[str, str]:
        env_var_copy = os.environ.copy()

        env_var_copy[EnvironmentVariable.ACCESS_KEY_ID_KEY] = credentials.access_key_id
        env_var_copy[EnvironmentVariable.SECRET_ACCESS_KEY_KEY] = credentials.secret_access_key
        env_var_copy[EnvironmentVariable.SESSION_TOKEN_KEY] = credentials.session_token

        if region_name is not None:
            env_var_copy[EnvironmentVariable.DEFAULT_REGION_KEY] = region_name

        return env_var_copy



from awsassume.assume_role_executor import AssumeRoleExecutor
from awsassume.data_models import AssumedRoleResponse, AssumedRoleResponseCredentials, Credentials
from awsassume.security_token_service import SecurityTokenService


class AssumeRoleNoCacheExecutor(AssumeRoleExecutor):

    def __init__(self, security_token_service: SecurityTokenService) -> None:
        self.security_token_service = security_token_service

    def execute(self) -> Credentials:
        response: AssumedRoleResponse = self.security_token_service.assume_role()

        response_credentials: AssumedRoleResponseCredentials = response['Credentials']
        credentials = Credentials(access_key_id=response_credentials['AccessKeyId'],
                                  secret_access_key=response_credentials['SecretAccessKey'],
                                  session_token=response_credentials['SessionToken'],
                                  expiration=response_credentials['Expiration'])

        return credentials



import os
import sys

from awsassume import __version__
from setuptools import setup


sys.path.append(f'{os.getcwd()}/awsassume')
sys.path.append(f'{os.getcwd()}/tests')

readme = None
with open('README.rst', 'r') as file:
    readme = file.read()

setup(name='awsassume',
      version=__version__,
      description='Execute AWS CLI commands after assuming role',
      long_description=readme,
      url='https://github.com/zulhilmizainuddin/aws-assume',
      author='Zulhilmi Mohamed Zainuddin',
      author_email='zulhilmi.zainuddin@outlook.com',
      license='MIT',
      packages=['awsassume'],
      classifiers=[
          'Development Status :: 3 - Alpha',
          'Intended Audience :: Developers',
          'Intended Audience :: System Administrators',
          'License :: OSI Approved :: MIT License',
          'Programming Language :: Python :: 3 :: Only',
          'Programming Language :: Python :: 3.6',
          'Environment :: Console'
      ],
      python_requires='>=3.6',
      install_requires=[
          'boto3>=1.6.16',
          'botocore>=1.9.16',
          'python-dateutil<2.7.0,>=2.1'
      ],
      tests_require=[
          'pytest>=3.5.0'
      ],
      setup_requires=[
          'pytest-runner>=4.2'
      ],
      entry_points={
          'console_scripts': [
              'awsassume=awsassume.__main__:console_entry'
          ]
      },
      test_suite='tests',
      zip_safe=False)



from datetime import datetime, timezone

from awsassume.assume_role_executor import AssumeRoleExecutor
from awsassume.response_cache_adapter import ResponseCacheAdapter
from awsassume.data_models import AssumedRoleResponse, AssumedRoleResponseCredentials, Credentials
from awsassume.security_token_service import SecurityTokenService


class AssumeRoleCacheExecutor(AssumeRoleExecutor):

    def __init__(self, security_token_service: SecurityTokenService, response_cache_adapter: ResponseCacheAdapter) -> None:

        self. security_token_service = security_token_service
        self.response_cache_adapter = response_cache_adapter

    def execute(self) -> Credentials:
        credentials: Credentials = None

        assumed_role_response: AssumedRoleResponse = self.response_cache_adapter.get_response_from_cache()
        if assumed_role_response is not None:

            expiration: datetime = assumed_role_response['Credentials']['Expiration']
            if expiration is not None and datetime.now(timezone.utc) < expiration:
                credentials = self.compose_credentials(assumed_role_response)
            else:
                assumed_role_response = self.newly_request_and_cache_assume_role()
                credentials = self.compose_credentials(assumed_role_response)
        else:
            assumed_role_response = self.newly_request_and_cache_assume_role()
            credentials = self.compose_credentials(assumed_role_response)

        return credentials

    def newly_request_and_cache_assume_role(self) -> AssumedRoleResponse:
        assumed_role_response: AssumedRoleResponse = self.security_token_service.assume_role()

        self.response_cache_adapter.set_response_to_cache(assumed_role_response)

        return assumed_role_response

    def compose_credentials(self, assumed_role_response: AssumedRoleResponse) -> Credentials:
        response_credentials: AssumedRoleResponseCredentials = assumed_role_response['Credentials']

        credentials = Credentials(access_key_id=response_credentials['AccessKeyId'],
                                  secret_access_key=response_credentials['SecretAccessKey'],
                                  session_token=response_credentials['SessionToken'],
                                  expiration=response_credentials['Expiration'])

        return credentials



from awsassume.data_models import AssumedRoleResponse, ResponseCacheArgs
from awsassume.response_cache import ResponseCache


class ResponseCacheAdapter(object):

    def __init__(self, response_cache_args: ResponseCacheArgs) -> None:
        self.response_cache_args = response_cache_args

        self.response_cache = ResponseCache()

    def set_response_to_cache(self, assumed_role_response: AssumedRoleResponse) -> None:
        self.response_cache.set_response_to_cache(self.response_cache_args, assumed_role_response)

    def get_response_from_cache(self) -> AssumedRoleResponse:
        return self.response_cache.get_response_from_cache(self.response_cache_args)

    def delete_cache_file(self) -> None:
        self.response_cache.delete_cache_file(self.response_cache_args)
